# Copyright 2006 Javier Fernandez-Sanguino <jfs@debian.org>
# This file contains all of the Debian-unique subroutines
# $Id: $

####################################################################
#
#  This module makes up the Debian specific API routines.
#  
####################################################################
#
#  Subroutine Listing:
#
#     &B_statoverride($):     Takes a file name and adds it current
#                             permissions to the dpkg-statoverride list
#
#####################################################################


###################################################################
#  &B_statoverride($file);
#    This subroutine is called after a file is modified (user, group
#    or permission ownership changes.  It will add the current permissions and
#    owner to the dpkg-statoverride lists so that the changes are preserved on
#    package upgrades.
#
#  Returns: 0 if statoverride was succesfully called (or was not called)
#           1 if the called of dpkg-statoverride returned an error
#
##################################################################
sub B_statoverride ($) {
    my $file = $_[0];

    # If the distribution is not Debian or there is no binary do nothing
    return 0 if &GetDistro !~ "^DB" ||  ! -e "/usr/sbin/dpkg-statoverride" ;
    
    # If the file does not exist return
    return 0 if ! -e $file ;
    # If the file is a symlink return
    # TODO - to cater for the alternatives' subsystem we would need to 
    # dereference the file it points to and apply dpkg-statoverride there
    return 0 if -l $file ;

    # Get the current stats of the file
    my $uid= (stat($file))[4];
    my $gid= (stat($file))[5];
    my $perm = (stat($file))[2];
    my $perm_formatted = sprintf("%05lo", $perm);
    $perm_formatted =~ s/^1//;

    
    my @override = ("/usr/sbin/dpkg-statoverride", "--force", "--add", "#$uid", "#$gid", "$perm_formatted", "$file");
    &B_log("ACTION","Setting permissions with dpkg-statoverride:".
            join(" ", @override)."\n");
    if ( system (@override) != 0 ) {
        &B_log("ERROR","Couldn't set a dpkg-statoverride for $file: $?\n");
        return 1;
    }
   return 0;
}


1;
